"""
https://www.digitalocean.com/community/tutorials/python-struct-pack-unpack
?: boolean
h: short
l: long
i: int
f: float
q: long long int

https://raspberrypi.stackexchange.com/questions/45266/how-to-read-a-long-int-via-i2c#45493

Versió de l'ESP32 a la carrier:
pi@raspberrypi:~/jorda $ i2cget -y 1 0x2A 0x06 w

20240203 - Jordi Binefa i Mar.
"""
import smbus
import time
import struct

address = 0x2A
cmd_read = 0x01
cmd_version = 0x06

def initI2C():
   bus = smbus.SMBus(1)
   return bus
   
def readLong(bus,cmd):
   block = bus.read_i2c_block_data(address, cmd) 
   n=struct.unpack('<l',bytes(block[:4]))[0]
   return n
   
def readShort(bus,cmd):
   block = bus.read_i2c_block_data(address, cmd) 
   n=struct.unpack('<h',bytes(block[:2]))[0]
   return n

"""
Reading ESP32's Carrier version:
pi@raspberrypi:~/jorda $ i2cget -y 1 0x2A 0x06 w
0x0100

"""
def readCarrierVersion(bus):
   version = readShort(bus,cmd_version)
   version = readShort(bus,cmd_version)
   return "%d.%d"%((version & 0xFF00)>>8,(version & 0x00FF))

def readEncoderPosition(bus):
   encoder = readLong(bus,cmd_read)
   encoder = readLong(bus,cmd_read)
   return encoder
   
def readEncoderPositionContinous(bus):
   encoder = readLong(bus,cmd_read)
   return encoder

def vSetupI2cRelayModule(bus,byAddr):
   bus.write_i2c_block_data(byAddr,0x04,[0x00,0x00])
   bus.write_i2c_block_data(byAddr,0x06,[0x00,0x00])

def vSetupI2cInputModule(bus,byAddr):
   bus.write_i2c_block_data(byAddr,0x04,[0xFF,0xFF])
   bus.write_i2c_block_data(byAddr,0x06,[0xBF,0xFF])
   
def vSetupI2c110vModule(bus,byAddr):
   bus.write_i2c_block_data(byAddr,0x04,[0xFF,0xFF])
   bus.write_i2c_block_data(byAddr,0x06,[0x3F,0x0F])

def siReadInputModule(bus,byAddr):
   block = bus.read_i2c_block_data(byAddr, cmd_read) 
   block = bus.read_i2c_block_data(byAddr, cmd_read) 
   n=struct.unpack('>H',bytes(block[:2]))[0]
   return n

def uiConverter(uiN):
  uiOut = 0x0000
  uiOut |= 0x0010 if (uiN & 0x0001) else 0x0000
  uiOut |= 0x0008 if (uiN & 0x0002) else 0x0000
  uiOut |= 0x0004 if (uiN & 0x0004) else 0x0000
  uiOut |= 0x0002 if (uiN & 0x0008) else 0x0000
  uiOut |= 0x0001 if (uiN & 0x0010) else 0x0000
  uiOut |= 0x1000 if (uiN & 0x0020) else 0x0000
  uiOut |= 0x0800 if (uiN & 0x0040) else 0x0000
  uiOut |= 0x0400 if (uiN & 0x0080) else 0x0000
  uiOut |= 0x0200 if (uiN & 0x0100) else 0x0000
  uiOut |= 0x0100 if (uiN & 0x0200) else 0x0000
  uiOut |= 0x2000 if (uiN & 0x0400) else 0x0000
  
  return uiOut
   
def vWriteRelayModule(bus, byAddr, uiV):
   byChipEnable = 0xA0
   bus.write_i2c_block_data(byAddr,0x02,[byChipEnable | (0xFF & uiConverter(uiV)),uiConverter(uiV) >> 8])

def byCabinButtonConverter(button,value):
   bOut = 0x00
   if button == 1:
      bOut = 0x81 if value == 1 else 0x80
   elif button == 2:
      bOut = 0x91 if value == 1 else 0x90
   elif button == 3:
      bOut = 0xA1 if value == 1 else 0xA0
   elif button == 4:
      bOut = 0xB1 if value == 1 else 0xB0
   elif button == 5:
      bOut = 0xC1 if value == 1 else 0xC0
   elif button == 6:
      bOut = 0xD1 if value == 1 else 0xD0

   return bOut

def vWriteCabinButtons(bus, byAddr, button, value):
   if button >= 1 and button <=6:
      bus.write_i2c_block_data(byAddr,0x02,[byCabinButtonConverter(button, value)])

if __name__ == "__main__":
   bus = initI2C()
   print("ESP32's Carrier version: %s"%readCarrierVersion(bus))
   time.sleep(.1)
   print("Encoder position: %d"%readEncoderPosition(bus))
   addrR = 0x20 # i2cdetect -y 1   #  0x20 = 0X20 + 0
   addrI = 0x27 #                  #  0x27 = 0x20 + 7  (jumper a l'aire = 1, o sigui OFF del microswitch)
   print("Testing relay and input module:")
   try:
      vSetupI2cInputModule(bus,addrI)
      whichOne = 0x0010
      vWriteRelayModule(bus,addrR,whichOne)
      print("Writing to relays module (0x%X): 0x%X"%(addrR,whichOne))
      time.sleep(.5)
      siReading = siReadInputModule(bus,addrI)
      print("Reading from input module (0x%X): %s"%(addrI,bin(siReading)[4::].zfill(14)))
      time.sleep(.5)
      whichOne = 0x0000
      vWriteRelayModule(bus,addrR,whichOne)
      print("Writing to relays module (0x%X): 0x%X"%(addrR,whichOne))
      time.sleep(.5)
      siReading = siReadInputModule(bus,addrI)
      print("Reading from input module (0x%X): %s"%(addrI,bin(siReading)[4::].zfill(14)))
   except:
      print("Address 0x%X doesn't exist"%addr)
   print("-----------------\nTesting relay module:")
   try:
      vSetupI2cRelayModule(bus,addrR)
      whichOne = 0x0001
      for i in range(11):
         vWriteRelayModule(bus,addrR,whichOne)
         print("Writing to relays module (0x%X): 0x%X"%(addrR,whichOne))
         whichOne <<= 1
         time.sleep(.5)
      whichOne = 0x0000
      vWriteRelayModule(bus,addrR,whichOne)
      print("Writing to relays module (0x%X): 0x%X"%(addrR,whichOne))
   except:
      print("Address 0x%X doesn't exist"%addr)
   
