"""
mqtt07.py

20240408 - Jordi Binefa
"""
from tecles00 import kbhit, getchar2
from carrierJorda import initI2C,readEncoderPosition,vSetupI2c110vModule,siReadInputModule
from carrierJorda import vSetupI2cRelayModule,vWriteRelayModule,vSetupI2cInputModule,vWriteCabinButtons
from microWorkers   import MicroWorkers
import json, os, serial, time, random, uuid
import paho.mqtt.subscribe as subscribe
import paho.mqtt.client as mqtt
import paho.mqtt.properties as properties

def boardAddress(addr):
    bAddr = 0x00
    if addr == "111":
        bAddr = 0x20
    elif addr == "110":
        bAddr = 0x21
    elif addr == "101":
        bAddr = 0x22
    elif addr == "100":
        bAddr = 0x23
    elif addr == "011":
        bAddr = 0x24
    elif addr == "010":
        bAddr = 0x25
    elif addr == "001":
        bAddr = 0x26
    elif addr == "000":
        bAddr = 0x27
    return bAddr

def cabinAddress(addr):
    bAddr = 0x00
    if addr == "111":
        bAddr = 0x37
    elif addr == "110":
        bAddr = 0x36
    elif addr == "101":
        bAddr = 0x35
    elif addr == "100":
        bAddr = 0x34
    elif addr == "011":
        bAddr = 0x33
    elif addr == "010":
        bAddr = 0x32
    elif addr == "001":
        bAddr = 0x31
    elif addr == "000":
        bAddr = 0x30
    return bAddr

def on_mqtt_message(clientId, userdata, message):
    global subscriptionTopics,bus
    msg = str(message.payload.decode("utf-8"))
    print('topic = %s' % message.topic,end="\n\r")
    print('payload = %s' % msg,end="\n\r")
    if message.topic == subscriptionTopics[0]: # topicSetup10Relays
        print("topicSetup10Relays at 0x%X address"%boardAddress(msg),end="\n\r")
        vSetupI2cRelayModule(bus,boardAddress(msg))
    if message.topic == subscriptionTopics[1]: # topicSetup14Inputs
        print("topicSetup14Inputs at 0x%X address"%boardAddress(msg),end="\n\r")
        vSetupI2cInputModule(bus,boardAddress(msg))
    if message.topic == subscriptionTopics[2]: # topicSetup6AcInputs
        print("topicSetup6AcInputs at 0x%X address"%boardAddress(msg),end="\n\r")
        vSetupI2c110vModule(bus,boardAddress(msg))
    if message.topic == subscriptionTopics[3]: # topicSet10Relays
        addr = boardAddress(msg.split(',')[0])
        decValue = int(msg.split(',')[1],2)
        print("topicSet10Relays with value 0x%X at 0x%X address"%(decValue,addr),end="\n\r")
        vWriteRelayModule(bus,addr,decValue)
    if message.topic == subscriptionTopics[4]: # topicGet14Inputs
        print("topicGet14Inputs at 0x%X address"%boardAddress(msg),end="\n\r")
        siReading = siReadInputModule(bus,boardAddress(msg))
        print("Reading from input module (0x%X): %s"%(boardAddress(msg),bin(siReading)[4::].zfill(14)),end="\n\r")
        mqttc.publish(topicSend14Inputs,bin(siReading)[4::].zfill(14))
    if message.topic == subscriptionTopics[5]: # topicGet6AcInputs
        print("topicGet6AcInputs at 0x%X address"%boardAddress(msg),end="\n\r")
        print("Valor 110v module (0x%X): %s"%(boardAddress(msg),bin(siReadInputModule(bus,boardAddress(msg)))[11::].zfill(6)),end="\n\r")
        mqttc.publish(topicSend6AcInputs,bin(siReadInputModule(bus,boardAddress(msg)))[11::].zfill(6))
    if message.topic == subscriptionTopics[6]: # topicGetCabinButtons
        print("topicGetCabinButtons at 0x%X address"%(cabinAddress(msg)),end="\n\r")
        readCabinButtons = bin(siReadInputModule(bus,cabinAddress(msg)))[2:].zfill(14)[:6]
        print("4)Valor botons (0x%X): %s"%(cabinAddress(msg),readCabinButtons),end="\n\r")
        mqttc.publish(topicSendCabinButtons,readCabinButtons)
    if message.topic == subscriptionTopics[7]: # topicSetCabinButtons
        addr = cabinAddress(msg.split(',')[0])
        button = int(msg.split(',')[1])
        value = int(msg.split(',')[2])
        print("addr: 0x%X, button: %d, value: %d"%(addr,button,value),end="\n\r")
        vWriteCabinButtons(bus,addr,button, value)  
    if message.topic == subscriptionTopics[8]: # topicGetEncoder
        encoderPosition = readEncoderPosition(bus)
        print("Valor encoder: ", encoderPosition,end="\n\r") 
        mqttc.publish(topicSendEncoder,encoderPosition)
    if message.topic == subscriptionTopics[9]: # topicSendTextFloor
        nStyle = int(msg.split(',')[0])
        textFloor = msg.split(',')[1]
        llistaText = []
        for c in textFloor:
            llistaText.append(ord(c))    
        data["msg"] = [nStyle] + llistaText
        data["n"] = 1 + len(llistaText)
        vSendCanBus(data)
        print("Missatge per trametre a la botonera de pis: ", msg,end="\n\r") 
        
def vMenu():
    # os.system("clear")
    print('msg: "%s". w: puja, a: esq, s: quiet, d: dre, z: baixa, n: nou missatge, e: encoder, 1: 110V module'%missatge)

def vSendCanBus(dt):
    dt=json.dumps(dt)
    print (dt)
    if ser.isOpen():
        ser.write(dt.encode('ascii'))
        ser.flush()
        """
        try:
            incoming = ser.readline().decode("utf-8")
            print (incoming)
        except Exception as e:
            print (e)
            pass
        """
    else:
        print ("opening error")

def serialReading(jobName,ser):
    while True:
        serialRx = ser.readline()
        if serialRx != b'':
            missatge = serialRx.decode("utf-8")
            print(missatge)
            try:
                mqttc.publish(topicSendStorey,missatge)
            except:
                print("No he pogut publicar: ",missatge)
        time.sleep(0.05)

ser  = serial.Serial("/dev/serial0", baudrate= 115200, 
    timeout=2.5, 
    parity=serial.PARITY_NONE, 
    bytesize=serial.EIGHTBITS, 
    stopbits=serial.STOPBITS_ONE
    )
data = {}
data["ID"] = 0x40 # CAN broadcasting
data["extd"] = 0
data["rtr"] = 0
# data["n"] = 4

macAddress = hex(uuid.getnode())[2::].upper()
topicSetup10Relays = "/"+macAddress+"/i2c/setup10Relays"
topicSetup14Inputs = "/"+macAddress+"/i2c/setup14Inputs"
topicSetup6AcInputs = "/"+macAddress+"/i2c/setup6AcInputs"
topicSet10Relays = "/"+macAddress+"/i2c/set10Relays"
topicGet14Inputs = "/"+macAddress+"/i2c/get14Inputs"
topicGet6AcInputs = "/"+macAddress+"/i2c/get6AcInputs"
topicGetCabinButtons = "/"+macAddress+"/i2c/getCabinButtons"
topicSetCabinButtons = "/"+macAddress+"/i2c/setCabinButtons"
topicGetEncoder = "/"+macAddress+"/i2c/getEncoder"
topicSendTextFloor = "/"+macAddress+"/can/sendTextFloor"

subscriptionTopics = [topicSetup10Relays,topicSetup14Inputs,topicSetup6AcInputs,topicSet10Relays,topicGet14Inputs,topicGet6AcInputs,topicGetCabinButtons,topicSetCabinButtons,topicGetEncoder,topicSendTextFloor]
print(subscriptionTopics)
topicSend14Inputs = "/"+macAddress+"/i2c/send14Inputs"
topicSend6AcInputs = "/"+macAddress+"/i2c/send6AcInputs"
topicSendCabinButtons = "/"+macAddress+"/i2c/sendCabinButtons"
topicSendEncoder = "/"+macAddress+"/i2c/sendEncoder"
topicSendStorey = "/"+macAddress+"/can/sendStorey"

mqttc = mqtt.Client('idClient-'+str(random.randrange(100000,999999)))
# mqttc.username_pw_set("usuari","contrasenya");
mqttc.connect("broker.emqx.io", 1883)
mqttc.on_message = on_mqtt_message
# mqttc.subscribe('topicName')
for tema in subscriptionTopics:
    mqttc.subscribe(tema)
mqttc.loop_start()

workers = MicroWorkers(workersCount=1)
workers.AddJob('serialReading',serialReading,arg=ser,onFinished=None)
# workers.AddJob('mqtt',mqtt,arg=None,onFinished=None)
time.sleep(0.05)
# os.system("clear")
addr110 = 0x22   #  0x22 = 0x20 + 2 (2 = només jumper al mig)
addrCabinButtons = 0x31 # 0x30 + 1 (jumper a A0)
try:
	bus = initI2C()
	vSetupI2c110vModule(bus,addr110)
except:
      print("Address 0x%X doesn't exist"%addr110)

'''
missatge = input("Missatge: ")
tramesa = []
for c in missatge:
    tramesa.append(ord(c))    
vMenu()
'''
while True:
    time.sleep(0.1)
    '''
    if kbhit():
        nextIteration,car = getchar2()
        if not nextIteration:
            break
        vMenu()
        if car == 'w':
            # data["msg"] = [1,82,80,105]
            data["msg"] = [1] + tramesa
            data["n"] = 1 + len(tramesa)
            vSendCanBus(data)
            print("'%s' pujant"%missatge)
        elif car == 'a':
            # data["msg"] = [4,82,80,105]
            data["msg"] = [4] + tramesa
            data["n"] = 1 + len(tramesa)
            vSendCanBus(data)
            print("'%s' cap a l'esquerra"%missatge)
        elif car == 's':
            data["msg"] = [0] + tramesa
            data["n"] = 1 + len(tramesa)
            vSendCanBus(data)            
            print("'%s' estàtic"%missatge)
        elif car == 'd':
            data["msg"] = [3] + tramesa
            data["n"] = 1 + len(tramesa)
            vSendCanBus(data)
            print("'%s' cap a la dreta"%missatge)
        elif car == 'z':
            data["msg"] = [2] + tramesa
            data["n"] = 1 + len(tramesa)
            vSendCanBus(data)
            print("'%s' baixa"%missatge)
        elif car == 'n':
            missatge = input("Missatge: ")
            vMenu()
            tramesa = []
            for c in missatge:
                tramesa.append(ord(c))
            # print(tramesa)
        elif car == 'e':
            print("Valor encoder: ", readEncoderPosition(bus))
            mqttc.publish('topicName','Escola del clot')
        elif car == '1':
            print("Valor 110v module (0x%X): %s"%(addr110,bin(siReadInputModule(bus,addr110))[11::].zfill(6)))
        else:
            print("?")
    '''
ser.close()
